<?php
/**
 * Helper functions to work with posts.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/admin/helpers
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * A class with several helper functions to work with posts.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/admin/helpers
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */
class Nelio_AB_Testing_Post_Helper {

	/**
	 * The single instance of this class.
	 *
	 * @since  5.0.0
	 * @access protected
	 * @var    Nelio_AB_Testing
	 */
	protected static $_instance;

	/**
	 * Returns the single instance of this class.
	 *
	 * @return Nelio_AB_Testing_Post_Helper the single instance of this class.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}//end if

		return self::$_instance;

	}//end instance()

	/**
	 * This function duplicates the given post.
	 *
	 * @param int    $src_post_id the post we want to duplicate.
	 * @param string $post_name   Optional. The post name we want to use for the duplicate.
	 *
	 * @return boolean|int the ID of the new post that's a duplicate of the given
	 *               one or 0 if an error occurred.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public function duplicate( $src_post_id, $post_name = false ) {

		$new_post_id = wp_insert_post(
			array(
				'post_author'  => 0,
				'post_title'   => 'Nelio A/B Testing',
				'post_content' => '',
				'post_excerpt' => '',
				'post_type'    => 'post',
				'post_status'  => 'nab_hidden',
				'post_name'    => ( $post_name ) ? $post_name : uniqid(),
			)
		);

		if ( empty( $new_post_id ) ) {
			return 0;
		}//end if

		$this->overwrite( $new_post_id, $src_post_id );

		return $new_post_id;

	}//end duplicate()

	/**
	 * This function overwites a post with the data from another post.
	 *
	 * @param int $dest_id the post we want to overwrite.
	 * @param int $src_id  the post whose data we want to use.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public function overwrite( $dest_id, $src_id ) {

		$this->overwrite_post_data( $dest_id, $src_id );
		$this->overwrite_post_meta( $dest_id, $src_id );
		$this->overwrite_post_terms( $dest_id, $src_id );

		/**
		 * Runs after overwriting a WordPress post with the data from another post.
		 *
		 * @param int $dest_id the destination post that has been overwritten using the source post info.
		 * @param int $src_id  the source post that's been duplicated.
		 *
		 * @since 5.0.0
		 */
		do_action( 'nab_overwrite_post', $dest_id, $src_id );

	}//end overwrite()

	/**
	 * This function overwites the data of a post (what's in the wp_posts table)
	 * with the data from another post.
	 *
	 * @param int $dest_id the post we want to overwrite.
	 * @param int $src_id  the post whose data we want to use.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function overwrite_post_data( $dest_id, $src_id ) {

		global $wpdb;
		$post = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM $wpdb->posts WHERE ID = %d",
				$src_id
			)
		);// db call ok; no-cache ok

		wp_update_post(
			array(
				'ID'             => $dest_id,
				'comment_status' => $post->comment_status,
				'menu_order'     => $post->menu_order,
				'ping_status'    => $post->ping_status,
				'post_author'    => $post->post_author,
				'post_type'      => $post->post_type,
				'post_title'     => $post->post_title,
				'post_content'   => $post->post_content,
				'post_excerpt'   => $post->post_excerpt,
				'post_parent'    => $post->post_parent,
				'post_password'  => $post->post_password,
			)
		);

	}//end overwrite_post_data()

	/**
	 * This function overwites the meta fields of a post with those from another post.
	 *
	 * @param int $dest_id the post whose meta fields we want to overwrite.
	 * @param int $src_id  the post whose meta fields we want to use.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function overwrite_post_meta( $dest_id, $src_id ) {

		$this->remove_old_metas( $dest_id );

		$metas = $this->get_metas( $src_id );
		foreach ( $metas as $meta ) {
			$this->insert_meta( $meta, $dest_id );
		}//end if

	}//end overwrite_post_meta()

	/**
	 * This function overwites the terms in which a post appears using those from another post.
	 *
	 * @param int $dest_id the post whose terms we want to overwrite.
	 * @param int $src_id  the post whose terms we want to use.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function overwrite_post_terms( $dest_id, $src_id ) {

		wp_delete_object_term_relationships( $dest_id, get_taxonomies() );

		$terms = wp_get_post_terms( $src_id );
		if ( ! is_wp_error( $terms ) ) {
			$terms = array_map(
				function( $term ) {
					return $term->name;
				},
				$terms
			);
			wp_set_post_terms( $dest_id, $terms );
		}//end if

		$categories = wp_get_post_categories( $src_id );
		if ( ! is_wp_error( $categories ) ) {
			wp_set_post_categories( $dest_id, $categories );
		}//end if

	}//end overwrite_post_terms()

	/**
	 * This function removes all the metas of a certain post (except those
	 * created by Nelio A/B Testing).
	 *
	 * @param int $post_id the post whose meta fields we want to remove.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function remove_old_metas( $post_id ) {

		global $wpdb;
		$wpdb->query(
			$wpdb->prepare(
				"DELETE FROM $wpdb->postmeta WHERE post_id = %d AND meta_key NOT LIKE %s",
				$post_id,
				$wpdb->esc_like( '_nab_' ) . '%'
			)
		);// db call ok; no-cache ok

	}//end remove_old_metas()

	/**
	 * This function retrieves all the metas of a certain post (except those
	 * created by Nelio A/B Testing), directly retrieved from the database using
	 * wpdb.
	 *
	 * @param int $post_id the post whose meta fields we want to retrieve.
	 *
	 * @return array all the metas of a certain post (except those created by
	 *         Nelio A/B Testing), directly retrieved from the database using
	 *         wpdb.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function get_metas( $post_id ) {

		global $wpdb;
		return $wpdb->get_results(
			$wpdb->prepare(
				"SELECT * FROM $wpdb->postmeta WHERE post_id = %d AND meta_key NOT LIKE %s",
				$post_id,
				$wpdb->esc_like( '_nab_' ) . '%'
			)
		);// db call ok; no-cache ok

	}//end get_metas()

	/**
	 * This function removes all the metas of a certain post (except those
	 * created by Nelio A/B Testing).
	 *
	 * @param object $meta    a meta field, as retrieved from the database.
	 * @param int    $post_id the post whose meta fields we want to remove.
	 *
	 * @since  5.0.0
	 * @access private
	 */
	private function insert_meta( $meta, $post_id ) {

		global $wpdb;
		$wpdb->insert(
			$wpdb->postmeta,
			array(
				'post_id'    => $post_id,
				'meta_key'   => $meta->meta_key,
				'meta_value' => $meta->meta_value,
			),
			array(
				'%d',
				'%s',
				'%s',
			)
		);// db call ok; slow query ok; no-cache ok

	}//end insert_meta()

}//end class
