<?php
/**
 * This class defines some helper functions to automatically flush cache
 * plugins when experiment statuses change.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/includes/utils
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * This class defines some helper functions to automatically flush cache
 * plugins when experiment statuses change.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/includes/utils
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */
class Nelio_AB_Testing_Cache_Manager {

	/**
	 * The single instance of this class.
	 *
	 * @since  5.0.0
	 * @access protected
	 * @var    Nelio_AB_Testing_Cache_Manager
	 */
	protected static $instance;

	/**
	 * Returns the single instance of this class.
	 *
	 * @return Nelio_AB_Testing_Cache_Manager the single instance of this class.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public static function instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}//end if

		return self::$instance;

	}//end instance()

	/**
	 * Hooks into WordPress.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public function init() {

		add_action( 'nab_start_experiment', [ $this, 'trigger_flush_all_caches' ] );
		add_action( 'nab_pause_experiment', [ $this, 'trigger_flush_all_caches' ] );
		add_action( 'nab_resume_experiment', [ $this, 'trigger_flush_all_caches' ] );
		add_action( 'nab_stop_experiment', [ $this, 'trigger_flush_all_caches' ] );

		add_action( 'nab_flush_all_caches', [ $this, 'flush_all_compatible_caches' ], 9 );

	}//end init()

	/**
	 * Triggers an event to flush all caches.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public function trigger_flush_all_caches() {

		/**
		 * Triggers a request to flush all compatible caches.
		 *
		 * By default, this action fires when an experiment is started, stopped,
		 * paused, or resumed. Hook into this action to add compatibility with
		 * your own cache plugin.
		 *
		 * @since 5.0.0
		 */
		do_action( 'nab_flush_all_caches' );

	}//end trigger_flush_all_caches()

	/**
	 * Flushes all caches from compatible chache plugins.
	 *
	 * @since  5.0.0
	 * @access public
	 */
	public function flush_all_compatible_caches() {

		$this->flush_wordpress_cache();
		$this->flush_w3_total_cache();

	}//end flush_all_compatible_caches()

	private function flush_wordpress_cache() {

		if ( ! function_exists( 'wp_cache_flush' ) ) {
			return;
		}//end if

		wp_cache_flush();

	}//end flush_wordpress_cache()

	private function flush_w3_total_cache() {

		if ( ! class_exists( 'W3_Plugin_TotalCacheAdmin' ) ) {
			return;
		}//end if

		$plugin = & w3_instance( 'W3_Plugin_TotalCacheAdmin' );
		$plugin->flush_all();

	}//end flush_w3_total_cache()

}//end class
