<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/public
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * The public-facing functionality of the plugin.
 */
class Nelio_AB_Testing_Public {

	protected static $instance;

	public static function instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}//end if

		return self::$instance;

	}//end instance()

	public function init() {

		$this->load_admin_helpers();

		add_action( 'init', [ $this, 'maybe_add_cookie_for_excluding_user_tracking' ] );
		add_action( 'wp_head', [ $this, 'add_kickoff_script' ], 1 );
		add_action( 'plugins_loaded', [ $this, 'init_split_testing' ] );
		add_action( 'set_current_user', [ $this, 'simulate_anonymous_visitor' ], 9999 );

		add_action(
			'plugins_loaded',
			function() {

				/**
				 * Initialized the public facet of the plugin.
				 *
				 * Fires right after WordPress’ `plugins_loaded` action with a low priority
				 * (so that other plugins can hook into `nab_public_init` during `plugins_loaded`).
				 *
				 * @since 5.0.0
				 */
				do_action( 'nab_public_init' );

			},
			9999
		);

	}//end init()

	public function maybe_add_cookie_for_excluding_user_tracking() {

		$is_visitor_tested = true;

		if ( is_super_admin() ) {
			$is_visitor_tested = false;
		}//end if

		if ( nab_is_current_user( 'editor' ) ) {
			$is_visitor_tested = false;
		}//end if

		/**
		 * Whether the user related to the current request should be tested or not.
		 *
		 * With this filter, you can decide if the current user participates in your running experiments or not.
		 * By default, all users are tested except those that have (at least) an `editor` role.
		 *
		 * **Notice.** Our plugin uses JavaScript to load alternative content. Be careful when limiting tests
		 * in PHP, as it’s possible that your cache or CDN ends up caching these limitations and, as a result,
		 * none of your visitors are tested.
		 *
		 * @param boolean $is_visitor_tested whether the user related to the current request should be tested or not.
		 *
		 * @since 5.0.0
		 */
		$is_visitor_tested = apply_filters( 'nab_is_visitor_tested', $is_visitor_tested );

		if ( ! $is_visitor_tested ) {
			setcookie( 'nabIsVisitorExcluded', 'true', time() + YEAR_IN_SECONDS, '/' );
		} else {
			setcookie( 'nabIsVisitorExcluded', 'true', time() - YEAR_IN_SECONDS, '/' );
		}//end if

	}//end maybe_add_cookie_for_excluding_user_tracking()

	public function add_kickoff_script() {

		if ( nab_is_split_testing_disabled() ) {
			return;
		}//end if

		ob_start();
		include nelioab()->plugin_path . '/public/kickoff.js';
		$content = ob_get_contents();
		ob_end_clean();

		$content = preg_replace( '/\/\/.*/', '', $content );
		$content = preg_replace( '/[\s\n]+/', ' ', $content );

		echo "<style type=\"text/css\" id=\"nab-alternative-loader-style\"></style>\n";
		echo '<script type="text/javascript">/* <![CDATA[ */';
		echo $content; // phpcs:ignore
		echo "/* ]]> */</script>\n";

	}//end add_kickoff_script()

	public function init_split_testing() {

		if ( nab_is_split_testing_disabled() ) {
			return;
		}//end if

		$aux = Nelio_AB_Testing_Runtime::instance();
		$aux->init();

		$aux = Nelio_AB_Testing_Alternative_Loader::instance();
		$aux->init();

	}//end init_split_testing()

	public function load_admin_helpers() {

		$aux = Nelio_AB_Testing_Alternative_Preview::instance();
		$aux->init();

		$aux = Nelio_AB_Testing_Css_Selector_Finder::instance();
		$aux->init();

		$aux = Nelio_AB_Testing_Heatmap_Renderer::instance();
		$aux->init();

		$aux = Nelio_AB_Testing_Quick_Experiment_Menu::instance();
		$aux->init();

	}//end load_admin_helpers()

	public function simulate_anonymous_visitor() {

		/**
		 * Simulates an anonymous visitor.
		 *
		 * When set to `true`, Nelio A/B Testing will set the current user
		 * to an anonymous (non-logged-in) user. This way, the web will look
		 * as if an anonymous users were browsing the site.
		 *
		 * @param boolean $anonymize whether the user should be set to an anonymous
		 *                           user or not. Default: `false`.
		 *
		 * @since 5.0.0
		 */
		if ( ! apply_filters( 'nab_simulate_anonymous_visitor', false ) ) {
			return;
		}//end if

		global $current_user;
		$current_user = new WP_User(); // phpcs:ignore

	}//end simulate_anonymous_visitor()

}//end class
