<?php
/**
 * This class adds the required scripts in the front-end to enable alternative loading.
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/public/helpers
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * This class adds the required scripts in the front-end to enable alternative loading.
 */
class Nelio_AB_Testing_Alternative_Loader {

	protected static $instance;
	const MAX_NUMBER_OF_POSSIBLE_COMBINATIONS = 24;

	public static function instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}//end if

		return self::$instance;

	}//end instance()

	public function init() {

		add_action( 'wp_head', [ $this, 'add_alternative_loader_script' ], 1 );
		add_action( 'wp_head', [ $this, 'maybe_add_canonical_link' ] );
		add_action( 'nab_relevant_experiments_loaded', [ $this, 'add_alternative_loading_hooks' ] );

	}//end init()

	public function add_alternative_loader_script() {

		$runtime  = Nelio_AB_Testing_Runtime::instance();
		$settings = Nelio_AB_Testing_Settings::instance();

		$alternative_loader_settings = array(
			'alternativeUrls'     => $this->get_alternative_urls(),
			'currentUrl'          => $runtime->get_current_url(),
			'gdprCookie'          => $runtime->get_gdpr_cookie(),
			'hideQueryArgs'       => $settings->get( 'hide_query_args' ),
			'isStagingSite'       => nab_is_staging(),
			'maxCombinations'     => self::MAX_NUMBER_OF_POSSIBLE_COMBINATIONS,
			'participationChance' => $settings->get( 'percentage_of_tested_visitors' ),
			'referrerParam'       => $runtime->get_referrer_param(),
			'untestedUrl'         => $runtime->get_untested_url(),
		);

		$asset   = include nelioab()->plugin_path . '/assets/dist/js/alternative-loader.asset.php';
		$version = $asset['version'];
		printf(
			"<script type=\"text/javascript\" src=\"%s\"></script>\n",
			esc_url( add_query_arg( 'version', $version, nelioab()->plugin_url . '/assets/dist/js/alternative-loader.js' ) )
		);

		echo '<script type="text/javascript">/* <![CDATA[ */';
		printf(
			'window.onNabAltLoaderInitAvailable( function() { nabloader.init( %s ); } );',
			wp_json_encode( $alternative_loader_settings )
		);
		echo "/* ]]> */</script>\n";

	}//end add_alternative_loader_script()

	public function maybe_add_canonical_link() {

		$settings = Nelio_AB_Testing_Settings::instance();
		if ( ! $settings->get( 'add_canonical_link' ) ) {
			return;
		}//end if

		$runtime       = Nelio_AB_Testing_Runtime::instance();
		$nab_query_arg = $runtime->get_nab_query_arg();
		if ( false === $nab_query_arg ) {
			return;
		}//end if

		printf(
			"<link rel=\"canonical\" href=\"%s\">\n",
			esc_url( $runtime->get_untested_url() )
		);

	}//end maybe_add_canonical_link()

	public function add_alternative_loading_hooks( $experiments ) {

		$runtime       = Nelio_AB_Testing_Runtime::instance();
		$nab_query_arg = $runtime->get_nab_query_arg();
		if ( false === $nab_query_arg ) {
			return;
		}//end if

		if ( ! is_array( $experiments ) ) {
			$experiments = [ $experiments ];
		}//end if

		foreach ( $experiments as $experiment ) {

			$experiment_type = $experiment->get_type();

			$control      = $experiment->get_alternative( 'control' );
			$alternatives = $experiment->get_alternatives();
			$alternative  = $alternatives[ $nab_query_arg % count( $alternatives ) ];

			/**
			 * Fires when a certain alternative is about to be loaded as part of a split test.
			 *
			 * Use this action to add any hooks that your experiment type might require in order
			 * to properly load the alternative.
			 *
			 * @param array  $alternative    attributes of the active alternative.
			 * @param array  $control        attributes of the control version.
			 * @param int    $experiment_id  experiment ID.
			 * @param string $alternative_id alternative ID.
			 *
			 * @since 5.0.0
			 */
			do_action( "nab_{$experiment_type}_load_alternative", $alternative['attributes'], $control['attributes'], $experiment->get_id(), $alternative['id'] );

		}//end foreach

	}//end add_alternative_loading_hooks()

	private function get_alternative_urls() {

		$url                 = $this->get_original_untested_url();
		$num_of_alternatives = $this->get_number_of_alternatives();

		if ( 2 > $num_of_alternatives ) {
			return array_fill( 0, self::MAX_NUMBER_OF_POSSIBLE_COMBINATIONS, $url );
		}//end if

		$result = [];
		for ( $i = 0; $i < self::MAX_NUMBER_OF_POSSIBLE_COMBINATIONS; ++$i ) {
			array_push( $result, add_query_arg( 'nab', $i % $num_of_alternatives, $url ) );
		}//end for
		return $result;

	}//end get_alternative_urls()

	private function get_number_of_alternatives() {

		$gcd = function( $n, $m ) use ( &$gcd ) {
			if ( 0 === $n || 0 === $m ) {
				return 1;
			}//end if
			if ( $n === $m && $n > 1 ) {
				return $n;
			}//end if
			return $m < $n ? $gcd( $n - $m, $n ) : $gcd( $n, $m - $n );
		};

		$lcm = function( $n, $m ) use ( &$gcd ) {
			return $m * ( $n / $gcd( $n, $m ) );
		};

		$experiments  = $this->get_experiments_that_load_alternative_content();
		$alternatives = array_unique(
			array_map(
				function( $experiment ) {
					return count( $experiment->get_alternatives() );
				},
				$experiments
			)
		);

		if ( empty( $alternatives ) ) {
			return 0;
		}//end if

		return array_reduce( $alternatives, $lcm, 1 );

	}//end get_number_of_alternatives()

	private function get_experiments_that_load_alternative_content() {

		$runtime     = Nelio_AB_Testing_Runtime::instance();
		$experiments = $runtime->get_relevant_running_experiments();

		return array_values(
			array_filter(
				$experiments,
				function( $experiment ) {

					$control         = $experiment->get_alternative( 'control' );
					$experiment_id   = $experiment->get_id();
					$experiment_type = $experiment->get_type();

					/**
					 *
					 * Whether the experiment should be excluded from the current request or not.
					 *
					 * @param boolean $is_excluded   whether the experiment should be excluded from the current request or not.
					 *                               Default: `false`.
					 * @param array   $control       original version.
					 * @param int     $experiment_id id of the experiment.
					 *
					 * @since 5.0.4
					 */
					return ! apply_filters( "nab_{$experiment_type}_exclude_experiment_from_loading", false, $control['attributes'], $experiment_id );

				}
			)
		);

	}//end get_experiments_that_load_alternative_content()

	private function get_original_untested_url() {

		$runtime      = Nelio_AB_Testing_Runtime::instance();
		$untested_url = $runtime->get_untested_url();

		if ( ! is_singular() ) {
			return $untested_url;
		}//end if

		$queried_object = get_queried_object_id();
		$experiment_id  = absint( get_post_meta( $queried_object, '_nab_experiment', true ) );
		$is_alternative = ! empty( $experiment_id );
		if ( ! $is_alternative ) {
			return $untested_url;
		}//end if

		$experiment  = nab_get_experiment( $experiment_id );
		$tested_post = absint( $experiment->get_tested_element() );
		if ( ! $tested_post ) {
			return $untested_url;
		}//end if

		if ( get_option( 'page_on_front', 0 ) === $tested_post ) {
			return home_url();
		}//end if

		return get_permalink( $tested_post );

	}//end get_original_untested_url()

}//end class
