<?php
/**
 * Some helper functions used during runtime
 *
 * @package    Nelio_AB_Testing
 * @subpackage Nelio_AB_Testing/public/helpers
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      5.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * Some helper functions used during frontend runtime.
 */
class Nelio_AB_Testing_Runtime {

	protected static $instance;

	private $relevant_experiments;
	private $relevant_heatmaps;

	private $current_url;

	public static function instance() {

		if ( is_null( self::$instance ) ) {

			self::$instance = new self();

			self::$instance->relevant_experiments = [];
			self::$instance->relevant_heatmaps    = [];
			self::$instance->current_url          = false;

		}//end if

		return self::$instance;

	}//end instance()

	public function init() {

		add_action( 'nab_public_init', [ $this, 'compute_relevant_experiments' ] );
		add_action( 'nab_public_init', [ $this, 'compute_relevant_heatmaps' ] );

	}//end init()

	public function get_relevant_running_experiments() {

		return $this->relevant_experiments;

	}//end get_relevant_running_experiments()

	public function get_relevant_running_heatmaps() {

		return $this->relevant_heatmaps;

	}//end get_relevant_running_heatmaps()

	public function get_current_url() {

		if ( empty( $this->current_url ) ) {
			$this->compute_current_url();
		}//end if
		return $this->current_url;

	}//end get_current_url()

	public function get_referrer_param() {

		/**
		 * Filters the query arg that retains the original referrer after performing a JavaScript redirection.
		 *
		 * Nelio A/B Testing loads the appropriate variant a visitor is supposed to see by
		 * performing a JavaScript redirection. As a result of this redirection, the original
		 * referrer is lost and the website owner can’t no longer know where the traffic is
		 * coming from.
		 *
		 * Use this filter to specify the query arg the plugin should use to retain this
		 * information. By default, it uses Google Analytics default arg: `utm_referrer`.
		 *
		 * If you don’t want to keep that information at all, set the value to `false`.
		 *
		 * @param boolean|string $param_name the name of the query arg that should keep track of the referrer or false otherwise.
		 *
		 * @since 5.0.0
		 */
		return apply_filters( 'nab_referrer_param', 'utm_referrer' );

	}//end get_referrer_param()

	public function get_untested_url() {

		return remove_query_arg( 'nab', $this->get_current_url() );

	}//end get_untested_url()

	public function get_gdpr_cookie() {

		/**
		 * Filters the name of the cookie that monitors GDPR acceptance.
		 *
		 * According to EU regulations and, in particular, the GDPR, visitors should be able to
		 * decide whether they want to be tracked by your website or not. If you need to comply
		 * to the GDPR, you can use this setting to specify the name of the cookie that must
		 * exist for tracking that visitor.
		 *
		 * By default, this setting is set to `false`, which means that all users will be tracked
		 * regardless of any other cookies.
		 *
		 * @param boolean|string $gdpr_cookie the name of the cookie that should exist if GDPR has
		 *                                    been accepted and, therefore, tracking is allowed.
		 *                                    Default: `false`.
		 *
		 * @since 5.0.0
		 */
		return apply_filters( 'nab_gdpr_cookie', false );

	}//end get_gdpr_cookie()

	public function compute_relevant_experiments() {

		$this->relevant_experiments = $this->filter_relevant_experiments( nab_get_running_experiments() );

		/**
		 * Fires after determining the list of relevant experiments.
		 *
		 * @param array $experiments list of relevant experiments.
		 *
		 * @since 5.0.0
		 */
		do_action( 'nab_relevant_experiments_loaded', $this->relevant_experiments );

	}//end compute_relevant_experiments()

	public function compute_relevant_heatmaps() {

		$untested_url = $this->get_untested_url();

		$this->relevant_heatmaps = array_filter(
			nab_get_running_heatmaps(),
			function ( $heatmap ) use ( $untested_url ) {

				if ( 'url' === $heatmap->get_tracking_mode() ) {
					$expected_url = $heatmap->get_tracked_url();
				} else {
					$expected_url = get_post_meta( $heatmap->get_tracked_post_id(), '_nab_permalink', true );
				}//end if

				return $this->do_urls_match_exactly( $untested_url, $expected_url );

			}
		);

		/**
		 * Fires after determining the list of relevant heatmaps.
		 *
		 * @param array $heatmaps list of relevant heatmaps.
		 *
		 * @since 5.0.0
		 */
		do_action( 'nab_relevant_heatmaps_loaded', $this->relevant_heatmaps );

	}//end compute_relevant_heatmaps()

	private function filter_relevant_experiments( $experiments ) {

		$relevant_experiments = array_filter(
			$experiments,
			function( $experiment ) {

				$scope = $experiment->get_scope();
				if ( empty( $scope ) ) {
					return true;
				}//end if

				foreach ( $scope as $rule ) {
					if ( $this->does_scope_rule_match( $rule['attributes'], $experiment ) ) {
						return true;
					}//end if
				}//end foreach

				return false;

			}
		);

		return array_values( $relevant_experiments );

	}//end filter_relevant_experiments()

	public function get_nab_query_arg( $url = false ) {

		if ( empty( $url ) ) {
			$url = $this->get_current_url();
		}//end if

		$query = wp_parse_args( wp_parse_url( $url, PHP_URL_QUERY ) );
		if ( ! isset( $query['nab'] ) ) {
			return false;
		}//end if

		return absint( $query['nab'] );

	}//end get_nab_query_arg()

	private function compute_current_url() {

		$url   = home_url( $this->get_clean_request_uri() );
		$query = wp_parse_args( wp_parse_url( $url, PHP_URL_QUERY ) );

		ksort( $query );
		$url = preg_replace( '/\?.*$/', '', $url );
		$url = add_query_arg( $query, $url );

		$url = remove_query_arg( 'nabforce', $url );
		$url = remove_query_arg( 'nabstaging', $url );
		if ( $this->get_referrer_param() ) {
			$url = remove_query_arg( $this->get_referrer_param(), $url );
		}//end if

		$nab = $this->get_nab_query_arg( $url );
		if ( false !== $nab ) {
			$url = remove_query_arg( 'nab', $url );
			$url = add_query_arg( 'nab', $nab, $url );
		}//end if

		$this->current_url = $url;

	}//end compute_current_url()

	private function get_clean_request_uri() {

		$request_uri             = $_SERVER['REQUEST_URI']; // phpcs:ignore
		$request_uri_in_home_url = preg_replace( '/^https?:\/\/[^\/]+/', '', home_url() );

		$request_uri             = '/' . ltrim( $request_uri, '/' );
		$request_uri_in_home_url = '/' . ltrim( $request_uri_in_home_url, '/' );

		if ( 0 !== strpos( $request_uri, $request_uri_in_home_url ) ) {
			return $request_uri;
		}//end if

		$request_uri = substr( $request_uri, strlen( $request_uri_in_home_url ) );
		if ( 0 < strlen( $request_uri ) && '/' !== $request_uri[0] ) {
			$request_uri = '/' . $request_uri;
		}//end if

		return $request_uri;

	}//end get_clean_request_uri()

	private function does_scope_rule_match( $rule, $experiment ) {

		$url = $this->get_untested_url();

		switch ( $rule['type'] ) {

			case 'tested-post':
				$post_id  = $experiment->get_tested_element();
				$post_url = get_post_meta( $post_id, '_nab_permalink', true );

				return $this->do_urls_match_exactly( $url, $post_url );

			case 'partial':
				return false !== stripos( $url, $rule['value'] );

			case 'exact':
				return $this->do_urls_match_exactly( $url, $rule['value'] );
		}//end switch

		return false;

	}//end does_scope_rule_match()

	private function do_urls_match_exactly( $actual_url, $expected_url ) {

		$actual_url   = strtolower( preg_replace( '/^[^:]+:\/\//', '', $actual_url ) );
		$expected_url = strtolower( preg_replace( '/^[^:]+:\/\//', '', $expected_url ) );

		$actual_args   = wp_parse_args( wp_parse_url( $actual_url, PHP_URL_QUERY ) );
		$expected_args = wp_parse_args( wp_parse_url( $expected_url, PHP_URL_QUERY ), $actual_args );

		ksort( $actual_args );
		ksort( $expected_args );

		$actual_url   = untrailingslashit( preg_replace( '/\?.*$/', '', $actual_url ) );
		$expected_url = untrailingslashit( preg_replace( '/\?.*$/', '', $expected_url ) );

		/**
		 * Whether to ignore query args when trying to match the current URL with a URL specified in an experiment scope.
		 *
		 * @param boolean $ignore whether to ignore query args when trying to match the URL with a URL specified in an experiment scope. Default: `false`.
		 *
		 * @since 5.0.0
		 */
		if ( ! apply_filters( 'nab_ignore_query_args_in_scope', false ) ) {
			$actual_url   = add_query_arg( $actual_args, $actual_url );
			$expected_url = add_query_arg( $expected_args, $expected_url );
		}//end if

		return $actual_url === $expected_url;

	}//end do_urls_match_exactly()

}//end class
